/**
 * SIMON GEHRIGs Slider
 *
 * jslint latedef:false
 * jslint browser:true
 */

sliders = new Object();

/**
 * SETS BACK DIMENSIONS IN GIVEN SLIDER ELEMENT
 * */
function dimensions( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );
    // (function( jQuery ) {
    /* Declare Variables*/
    slider.bannergroupwidth = null;
    slider.bannerheight = null;
    slider.banneroffset = null;

    /* LOGGING to check if running */
    console.log( slider.bannergroup.length );
    console.log( slider.banneritems.length );

    /* Set Banner Group Width */
    if ( null === slider.bannergroupwidth ) {
        slider.bannergroupwidth = slider.bannergroup[ 0 ].clientWidth;
    }
    /* Set Banner Item Width */
    if ( null === slider.banneritemwidth ) {
        slider.banneritemwidth = slider.banneritems[ 0 ].clientWidth;
    }
    /* CALCULATE BANNER DIMENSIONS*/
    if ( null === slider.bannerheight ) {
        slider.bannerheight = slider.banneritemwidth / slider.slideraspect;
    }
    if ( null === slider.banneroffset ) {
        slider.banneroffset = ( slider.bannergroupwidth - slider.banneritemwidth ) / 2;
    }
    /*}( jQuery ) );*/

}

/**
 * Set up new Slider
 * */
function newSlider( identifier, xaspect, yaspect, groupselector, itemselector ) {
    "use strict";
    /* Declare Variables */
    var ret = null;
    var newslider = new Object();
    /*GROUP & ITEMS*/
    var bannergroup = jQuery( groupselector );
    var banneritems = jQuery( itemselector );
    /*ORDERING*/
    var activeitem = 0;
    var orderarray = new Array();
    /*WIDTHS, HEIGHTS & OFFSETS*/
    var slideraspect = xaspect / yaspect;
    var bannergroupwidth = null;
    var banneritemwidth = null;
    var bannerheight = null;
    var banneroffset = null;
    /*TIMER & EVENTS*/
    var bannerautoslide = null;
    var ready;
    var resize;
    var afterresize;
    var resizetimeout;

    /*

     LOGIC

     */
    /*
     Find an existing Slider
     in case we have only the identifier given
     */
    if ( identifier && ! xaspect && ! yaspect && ! groupselector && ! itemselector ) {
        ret = sliders[ identifier ];
        if ( null === ret ) {
            window.console.error( "no such slider found: " + identifier );
        }
    }
    /*
     Open up new Slider
     in case all of the arguments are passed
     */
    else if ( identifier && xaspect && yaspect && groupselector && itemselector ) {
        if ( bannergroup.length !== 0 && banneritems.length !== 0 ) {
            window.console.log( "new slider with identifier: " + identifier );
            /*
             *
             * EVENTHANDLING
             *
             * */
            /*
             WHEN DOCUMENT IS READY
             */
            ready = (
                function( jQuery ) {
                    bannergroup.ready(
                        function() {
                            /*Log the execution of this event*/
                            console.log( "Load Event triggered for: " + identifier );
                            /*Turn off transitions*/
                            notransitions( identifier );
                            /*Order the Slider Items*/
                            order( identifier );
                            /*Start autoplay*/
                            start( identifier );
                            /*Get the dimensions of the Slider*/
                            dimensions( identifier );
                            /*Execute the slider logic*/
                            banner( identifier );
                            /*Turn on trasitions again*/
                            yestransitions( identifier );
                        }
                    );
                }( jQuery ) );

            /*IF WINDOW GETS RESIZED*/
            resize = jQuery( window ).resize( function() {
                /*Log the execution of this event*/
                console.log( "Resize Event triggered for: " + identifier );
                /* Reset all values */
                resetcss( identifier );
                /* Turn transitions off
                * this is necessary to keep
                * the timeout low.
                * otherwise it will be faster than
                * the transition and the calculation
                * will return wrong values */
                notransitions( identifier );
                /* Clear te timeaout if it is already set (not timed out) */
                if ( resizetimeout ) {
                    clearTimeout( resizetimeout );
                }
                /* Set the timeout */
                resizetimeout = setTimeout( function() {
                    jQuery( this ).trigger( 'resizeEnd' + identifier );
                }, 200 );
            } );

            /* AFTER WINDOW IS RESIZED */
            afterresize = jQuery( window ).bind( "resizeEnd" + identifier, function() {
                /*Log the execution of this event*/
                console.log( "After Resize Event triggered for: " + identifier );
                /* Get the newly calculated dimensions
                * this is the cause for the timeout,
                * since we need to get the actual size
                * of certain elements AFTER resizing. */
                dimensions( identifier );
                /* Fire the Banner functions to trigger resizing */
                banner( identifier );
                /* Turn transitions on again */
                yestransitions( identifier );
            } );
        }
        /*
         *
         * END CONFIGURATION DEFAULTS
         *
         */
        /*
         ADD TO SLIDER OBJECT
         */
        newslider.identifier = identifier;
        newslider.groupselector = groupselector;
        newslider.itemselector = itemselector;
        newslider.bannergroup = bannergroup;
        newslider.banneritems = banneritems;
        newslider.activeitem = activeitem;
        newslider.xaspect = xaspect;
        newslider.yaspect = yaspect;
        newslider.slideraspect = slideraspect;
        newslider.bannergroupwidth = bannergroupwidth;
        newslider.banneritemwidth = banneritemwidth;
        newslider.bannerheight = bannerheight;
        newslider.banneroffset = banneroffset;
        newslider.bannerautoslide = bannerautoslide;
        newslider.ready = ready;
        newslider.resize = resize;
        newslider.resizetimeout = resizetimeout;
        newslider.afterrezize = afterresize;
        newslider.orderarray = orderarray;
        /**/
        sliders[ identifier ] = newslider;
        ret = newslider;
    }
    /* Error because no slider was found
     * in case there is nothing or incomplete data given
     * */
    else {
        window.console.error( "Either declare whole new slider and pass all arguments to create new instance " );
        window.console.error( "or only pass the identifier to get an existing slider" );
    }
    return ret;
}

/**
 * Reset Element CSS */
function resetcss( sliderid ) {
    "use strict";
    /* Declare Variables */
    var i = 0;
    /* Get the Slider */
    var slider = newSlider( sliderid );
    var banneritems = jQuery( slider.itemselector );
    /* Get jQuery Objects */
    var linksinitems = jQuery( slider.itemselector + " a" );
    var bannercontrolsleft = jQuery( slider.groupselector + " .bannercontrols .left" );
    var bannercontrolsright = jQuery( slider.groupselector + " .bannercontrols .right" );

    /* UNSET CSS Values */
    if ( slider.bannergroup ) {
        slider.bannergroup.css( {
            height: ""
        } );
    }

    if ( linksinitems ) {
        linksinitems.css( {
            width: "",
            height: ""
        } );
    }

    if ( bannercontrolsleft ) {
        bannercontrolsleft.css( {
            left: "",
            top: "",
            width: "",
            height: ""
        } );
    }

    if ( jQuery( bannercontrolsright ) ) {
        jQuery( bannercontrolsright ).css( {
            left: "",
            top: "",
            width: "",
            height: ""
        } );
    }
    if ( banneritems ) {
        /*Log the length of the Array*/
        console.log( "banneritems:" + banneritems.length );
        /*Loop through the Items*/
        for ( i = 0 ; i < banneritems.length ; i += 1 ) {

            var banneritem = jQuery( slider.itemselector + ":eq(" + i + ")" );

            slider.banneritems[ i ].style.width = null;
            slider.banneritems[ i ].style.height = null;
            slider.banneritems[ i ].style.left = null;
        }
    }

    slider.bannergroupwidth = null;
    slider.banneritemwidth = null;
    slider.banneroffset = null;
    slider.bannerheight = null;
}

/**
 * Jump to specific Banneritem
 * */
function jump( sliderid, item ) {
    "use strict";
    (function( jQuery ) {
        var slider = newSlider( sliderid );

        // STOP autoslider
        pause( sliderid );

        // JUMP to chosen Item
        slider.activeitem = item;

        // Reorder
        order( sliderid );
    }( jQuery ) );
}

/**
 * Button Left Function
 * */
function buttonL( sliderid ) {
    "use strict";

    pause( sliderid );
    scrollL( sliderid );
}

/**
 * Button Right Function
 * */
function buttonR( sliderid ) {
    "use strict";

    pause( sliderid );
    scrollR( sliderid );
}

/**
 * Scroll through the Banneritems
 * to the left
 * */
function scrollL( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );

    // GET Number of Items in the bannergroup
    slider.activeitem += 1;

    if ( slider.activeitem >= slider.banneritems.length ) {
        slider.activeitem = slider.activeitem - slider.banneritems.length;
    }

    order( slider.identifier );
}

/**
 * Scroll through the Banneritems
 * to the Right
 * */
function scrollR( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );

    // GET Number of Items in the bannergroup
    slider.activeitem -= 1;

    if ( slider.activeitem < 0 ) {
        slider.activeitem = slider.activeitem + slider.banneritems.length;
    }

    order( slider.identifier );
}

/**
 * Pause automatic Sliding /
 * Pause Button Function
 * */
function pause( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );

    // Pause Button Press
    if ( null !== typeof slider.bannerautoslide ) {
        window.clearInterval( slider.bannerautoslide );
    }
}

/**
 * Start automatic Sliding
 * */
function start( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );

    // Pause to avoid multiple Intervals
    pause( sliderid );

    // Scroll on Time
    slider.bannerautoslide = window.setInterval( function() {
        scrollL( sliderid );
    }, 7500 );
}

/**
 * Play Button Function
 * */
function play( sliderid ) {
    "use strict";

    // Play Button Press
    scrollR( sliderid );
    start( sliderid );
}

// TODO Set images as background-images

/**
 * Turn CSS-Transitions off
 * */
function notransitions( sliderid ) {
    "use strict";
    /*Remove the transitions set in css
     by overwriting them in Element*/
    var slider = newSlider( sliderid );

    slider.bannergroup.css( {
        transition: "none"
    } );
    jQuery( slider.groupselector + " *" ).css( {
        transition: "none"
    } );
}

/**
 * Turn CSS-Transitions on
 * */
function yestransitions( sliderid ) {
    "use strict";
    /*Remove the transitions set in Element
     to make the ones set in css work again*/
    var slider = newSlider( sliderid );

    slider.bannergroup.css( {
        transition: ""
    } );
    jQuery( slider.groupselector + " *" ).css( {
        transition: ""
    } );
}

/**
 * Reorder the items (Slide)
 * */
function order( sliderid ) {
    "use strict";

    var slider = newSlider( sliderid );
    /*Counters*/
    var i;
    var j;

    /*Variables*/
    slider.banneritems = jQuery( slider.itemselector );

    /*Calculate the center Position*/
    slider.activepos = Math.round( slider.banneritems.length / 2 );

    /*Get all item Numbers in an Array of the positions*/
    for ( i = 0 ; i < slider.banneritems.length ; i += 1 ) {
        slider.orderarray[ i ] = i;
    }

    /*Rotate the Array by the Number of the middle position
     plus the number of the active item*/
    for ( j = 0 ; j < slider.activepos + slider.activeitem ; j += 1 ) {
        slider.orderarray.unshift( slider.orderarray.pop() );
    }

    /*Execute the banner function */
    banner( sliderid );
}

/**
 * BANNER RELATED FUNCTIONS (main function)
 * */
function banner( sliderid ) {
    "use strict";
    var slider = newSlider( sliderid );
    ( function( jQuery ) {

        /* -----------------------------------------------------------------------
         THE BANNER SLIDER
         -----------------------------------------------------------------------*/
        /*
         Get the Control Elements
         */
        var bannerjumpwrapper = jQuery( slider.groupselector + " .bannerjump-wrapper" );
        var bannercontrols = jQuery( slider.groupselector + " .bannercontrols" );
        var bannerautoplay = jQuery( slider.groupselector + " .bannerautoplay" );
        /*
         Make the banner arraylength more comfortable to calculate in the loop
         */
        var bannerarraylength = slider.banneritems.length - 1;
        /*JQUERY Objects*/
        /*With links in Items*/
        var linksinitems = jQuery( slider.itemselector + " a" );
        /*With bannercontriols*/
        var bannercontrolsleft;
        var bannercontrolsright;
        /*Iframes in Items*/
        var iframes = jQuery( slider.itemselector + " iframe" );
        var iframe;
        /*Counter*/
        var i;
        /*
         Variables for
         the loop
         */
        var banneritemnr;
        var posnr;
        var banneritem;
        var posoffset = ( slider.activepos - 1 ) - posnr;
        var left = ( posoffset * slider.banneritemwidth ) + slider.banneroffset;
        var state = null;
        var removal = null;
        /*
         SET activeelement to first element if not set
         */
        if ( ! slider.activeitem ) {
            slider.activeitem = 0;
        }
        /*-----------------------------------------------------------------------
         HTML
         -----------------------------------------------------------------------

         REMOVE JUMPER AND LEFT/RIGHT CONTROLS
         ------------------------------------------------------------------------
         */
        bannerjumpwrapper.detach();
        bannercontrols.detach();
        bannerautoplay.detach();

        if ( bannerarraylength > 0 ) {
            /*ADD THE CONTROLS
             ------------------------------------------------------------------------
             ADD the Jumper-wrapper*/
            slider.bannergroup.append( "<div class=\"bannerjump-wrapper\"></div>" );
            /*TODO Jumps to <a>-top by clicking left/right arrows*/
            /*ADD the left/right start/stop controls*/
            slider.bannergroup
                .append(
                    "<div class=\"bannerautoplay\">" +
                    "<a class=\"pause\" onclick=\"pause( '" + slider.identifier + "' )\"></a>" +
                    "<a class=\"play\" onclick=\"play( '" + slider.identifier + "' )\"></a> " +
                    "</div> " +
                    "<div class=\"bannercontrols\">" +
                    "<a class=\"right\" onclick=\"buttonR( '" + slider.identifier + "' )\" >" +
                    "<div class=\"iconwrapper\">" +
                    "<span class=\"circle\">" +
                    "<span class=\"arrow\">" +
                    "</span>" +
                    "</span>" +
                    "</div>" +
                    "</a>" +
                    "<a class=\"left\" onclick=\"buttonL( '" + slider.identifier + "' )\" >" +
                    "<div class=\"iconwrapper\">" +
                    "<span class=\"circle\">" +
                    "<span class=\"arrow\">" +
                    "</span>" +
                    "</span>" +
                    "</div>" +
                    "</a>" +
                    "</div>" );

        }

        // SET BANNER DIMENSIONS
        // --------------------------------------------------------------------
        slider.bannergroup.css( {
            height: slider.bannerheight
        } );

        linksinitems.css( {
            width: slider.banneritemwidth,
            height: slider.bannerheight
        } );

        /*
         * If only one Item in Array, do not display the controls
         * */
        if ( bannerarraylength > 0 ) {
            /*BANNER CONTROLS
             --------------------------------------------------------------------------


             SET LEFT/RIGHT DIMENSIONS*/
            bannercontrolsleft = jQuery( slider.groupselector + " .bannercontrols .left" );
            bannercontrolsright = jQuery( slider.groupselector + " .bannercontrols .right" );

            bannercontrolsright.css( {
                position: "absolute",
                left: 0,
                top: 0,
                width: slider.banneroffset,
                height: slider.bannerheight,
                "z-index": "10000"
            } );

            bannercontrolsleft.css( {
                position: "absolute",
                right: 0,
                top: 0,
                width: slider.banneroffset,
                height: slider.bannerheight,
                "z-index": "10000"
            } );

        }
        /*SET THE BANNERITEMS"POSITIONS
         --------------------------------------------------------------*/
        /*LOOP THROUGH ALL ITEMS*/
        for ( i = 0 ; i <= ( bannerarraylength ) ; i += 1 ) {
            banneritemnr = i;
            posnr = ( slider.banneritems.length - 1 ) - slider.orderarray[ i ];
            /*Get the item for position $i*/
            banneritem = jQuery( slider.itemselector + ":eq(" + banneritemnr + ")" );
            /*Get the position-offset in Layout
             position 1 is the middle all others are
             aligned around it:
             position 2 is after 1 and the last position
             position n is before it*/
            posoffset = ( slider.activepos - 1 ) - posnr;
            /*Calculate the offset based on the position*/
            left = ( posoffset * slider.banneritemwidth ) + slider.banneroffset;
            /*Variables for state and removal classes*/
            state = null;
            removal = null;
            /*Set CLASSES for styling*/
            if ( slider.activeitem === i ) {
                state = "active";
                removal = "inactive";
            } else if ( ( slider.activeitem - 1 ) === i || slider.activeitem + 1 === i ) {
                state = "inactive visible";
                removal = "active";
            } else if ( slider.activeitem === bannerarraylength && 0 === i ) {
                state = "inactive visible";
                removal = "active";
            } else if ( 0 === slider.activeitem && i === bannerarraylength ) {
                state = "inactive visible";
                removal = "active";
            } else {
                state = "inactive";
                removal = "active visible";
            }

            // SET THE VALUES CALCULATED IN THE LOGIC
            banneritem.css( {
                "left": left,
                "width": slider.banneritemwidth,
                "height": slider.bannerheight
            } );

            banneritem.addClass( state );
            banneritem.removeClass( removal );

            //
            // JUMP CIRCLES
            if ( bannerarraylength > 0 ) {

                // CONSTRUCT THE JUMP-ICONS
                jQuery( slider.groupselector + " .bannerjump-wrapper" ).append(
                    "<a class=\"bannerjump bannerjump-" + i + "\" onclick=\"jump('" + slider.identifier + "', " + i + ")\"></a>"
                );

                // Make the active jumper colored different
                if ( slider.activeitem === i ) {
                    jQuery( slider.groupselector + " .bannerjump-" + ( i ) ).addClass( "active" );
                }
            }

        } // End for()

        //prevent banner iframes from getting too large
        for ( i = 0 ; i < iframes.length ; i += 1 ) {
            iframe = iframes[ i ];
            iframe.removeAttribute( "height" );
            iframe.removeAttribute( "width" );
        }

    }( jQuery ) );
}

/**
 * LIST OF ALL FUNCTIONS TRIGGERED ABOVE
 * */
function maintenance( sliderid ) {
    "use strict";
    dimensions( sliderid );
    banner( sliderid );
}
